package ca.recrown.islandsurvivalcraft;

import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.HandlerList;
import org.bukkit.event.Listener;
import org.bukkit.event.server.ServerLoadEvent;
import org.bukkit.generator.ChunkGenerator;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import ca.recrown.islandsurvivalcraft.interaction.commands.CommandProcessor;
import ca.recrown.islandsurvivalcraft.interaction.items.VariedItemManager;
import ca.recrown.islandsurvivalcraft.world.WorldInfoManager;
import ca.recrown.islandsurvivalcraft.world.generation.GeneratorModes;

public class IslandSurvivalCraft extends JavaPlugin implements Listener {
    private PluginManager pluginManager;
    private WorldInfoManager worldInfoManager;
    private CommandProcessor commandProcessor;
    private VariedItemManager variedItemManager;

    public IslandSurvivalCraft() {
        worldInfoManager = new WorldInfoManager();
        variedItemManager = new VariedItemManager(this);
        commandProcessor = new CommandProcessor(this);
    }
    
    @Override
    public void onEnable() {
        pluginManager = getServer().getPluginManager();
        pluginManager.registerEvents(worldInfoManager, this);
        pluginManager.registerEvents(variedItemManager, this);
        pluginManager.registerEvents(this, this);
        super.onEnable();
    }
    
    @EventHandler(priority = EventPriority.MONITOR)
    public void onReady(ServerLoadEvent e) {
        variedItemManager.loadAllItems();
    }

    @Override
    public void onDisable() {
        HandlerList.unregisterAll(worldInfoManager);
        HandlerList.unregisterAll(variedItemManager);
        HandlerList.unregisterAll((Listener) this);
        variedItemManager.unloadAllItems();
        super.onDisable();
    }

    @Override
    public ChunkGenerator getDefaultWorldGenerator(String worldName, String id) {
        GeneratorModes gID = null;
        try {
            gID = GeneratorModes.valueOf(id);
        } catch (NullPointerException | IllegalArgumentException e) {
            gID = GeneratorModes.UNIQUE;
        }
        return worldInfoManager.getChunkGenerator(worldName, gID);
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        return commandProcessor.onCommand(sender, command, label, args);
    }



    /**
     * @return the world information manager.
     */
    public WorldInfoManager getWorldInfoManager() {
        return worldInfoManager;
    }

}
