package ca.recrown.islandsurvivalcraft.interaction.commands.runnables;

import org.bukkit.command.CommandSender;

import ca.recrown.islandsurvivalcraft.IslandSurvivalCraft;

public interface CommandRunnable {
    /**
     * @return a string that gives users an idea of what this command does. Should be short and not include details on arguments.
     */
    public String getDescription();

    /**
     * @return A more detailed description. Doesn't need to contain information already present in normal description as will print with that one. This string should describe proper usage arguments required.
     */
    public String getDetailedDescription();

    /**
     * Invokes this particular command.
     * Should check for correct arguments.
     * @param sender The sender of this command.
     * @param args The arguments given for this command. May be length of 0, in which case there were no arguments.
     * @return True if the execution of this command was a success, and false to display the help message for this command.
     */
    public boolean invoke(CommandSender sender, String[] args);

    /**
     * Initializes the command. Is run the first time this command is run.
     * @param islandSurvivalCraft
     */
    public void initialize(IslandSurvivalCraft islandSurvivalCraft);

    /**
     * 
     * @return True if on the next call, should initialize.
     */
    public boolean requiresInitialization();
}